@@ -1,5 +1,5 @@
 use proc_macro2::Ident;
-use syn::{Attribute, Type};
+use syn::{Attribute, Expr, Type};
 
 #[derive(Debug, Clone)]
 pub struct NapiFn {
@@ -15,6 +15,10 @@ pub struct NapiFn {
   pub vis: syn::Visibility,
   pub parent: Option<Ident>,
   pub strict: bool,
+  pub js_mod: Option<String>,
+  pub ts_args_type: Option<String>,
+  pub ts_return_type: Option<String>,
+  pub comments: Vec<String>,
 }
 
 #[derive(Debug, Clone)]
@@ -54,6 +58,8 @@ pub struct NapiStruct {
   pub fields: Vec<NapiStructField>,
   pub is_tuple: bool,
   pub kind: NapiStructKind,
+  pub js_mod: Option<String>,
+  pub comments: Vec<String>,
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -70,6 +76,7 @@ pub struct NapiStructField {
   pub ty: syn::Type,
   pub getter: bool,
   pub setter: bool,
+  pub comments: Vec<String>,
 }
 
 #[derive(Debug, Clone)]
@@ -78,6 +85,8 @@ pub struct NapiImpl {
   pub js_name: String,
   pub items: Vec<NapiFn>,
   pub task_output_type: Option<Type>,
+  pub js_mod: Option<String>,
+  pub comments: Vec<String>,
 }
 
 #[derive(Debug, Clone)]
@@ -85,6 +94,8 @@ pub struct NapiEnum {
   pub name: Ident,
   pub js_name: String,
   pub variants: Vec<NapiEnumVariant>,
+  pub js_mod: Option<String>,
+  pub comments: Vec<String>,
 }
 
 #[derive(Debug, Clone)]
@@ -93,3 +104,19 @@ pub struct NapiEnumVariant {
   pub val: i32,
   pub comments: Vec<String>,
 }
+
+#[derive(Debug, Clone)]
+pub struct NapiConst {
+  pub name: Ident,
+  pub js_name: String,
+  pub type_name: Type,
+  pub value: Expr,
+  pub js_mod: Option<String>,
+  pub comments: Vec<String>,
+}
+
+#[derive(Debug, Clone)]
+pub struct NapiMod {
+  pub name: Ident,
+  pub js_name: String,
+}
