@@ -2,7 +2,7 @@ use proc_macro2::{Ident, Span, TokenStream};
 use quote::ToTokens;
 
 use crate::{
-  codegen::{get_intermediate_ident, get_register_ident},
+  codegen::{get_intermediate_ident, get_register_ident, js_mod_to_token_stream},
   BindgenResult, CallbackArg, FnKind, FnSelf, NapiFn, NapiFnArgKind, TryToTokens,
 };
 
@@ -245,6 +245,8 @@ impl NapiFn {
     let js_name = &self.js_name;
 
     if let Some(ty) = &self.ret {
+      let ty_string = ty.into_token_stream().to_string();
+      let is_return_self = ty_string == "& Self" || ty_string == "&mut Self";
       if self.kind == FnKind::Constructor {
         if self.is_ret_result {
           quote! { cb.construct(#js_name, #ret?) }
@@ -262,6 +264,8 @@ impl NapiFn {
           quote! {
             <#ty as napi::bindgen_prelude::ToNapiValue>::to_napi_value(env, #ret)
           }
+        } else if is_return_self {
+          quote! { #ret.map(|_| cb.this) }
         } else {
           quote! {
             match #ret {
@@ -273,6 +277,8 @@ impl NapiFn {
             }
           }
         }
+      } else if is_return_self {
+        quote! { Ok(cb.this) }
       } else {
         quote! {
           <#ty as napi::bindgen_prelude::ToNapiValue>::to_napi_value(env, #ret)
@@ -290,36 +296,41 @@ impl NapiFn {
       quote! {}
     } else {
       let name_str = self.name.to_string();
-      let js_name = &self.js_name;
+      let js_name = format!("{}\0", &self.js_name);
       let name_len = js_name.len();
       let module_register_name = get_register_ident(&name_str);
       let intermediate_ident = get_intermediate_ident(&name_str);
-
+      let js_mod_ident = js_mod_to_token_stream(self.js_mod.as_ref());
+      let cb_name = Ident::new(
+        &format!("__register__fn__{}_callback__", name_str),
+        Span::call_site(),
+      );
       quote! {
+        unsafe fn #cb_name(env: napi::bindgen_prelude::sys::napi_env) -> napi::bindgen_prelude::Result<napi::bindgen_prelude::sys::napi_value> {
+          let mut fn_ptr = std::ptr::null_mut();
+
+          napi::bindgen_prelude::check_status!(
+            napi::bindgen_prelude::sys::napi_create_function(
+              env,
+              #js_name.as_ptr() as *const _,
+              #name_len,
+              Some(#intermediate_ident),
+              std::ptr::null_mut(),
+              &mut fn_ptr,
+            ),
+            "Failed to register function `{}`",
+            #name_str,
+          )?;
+
+          Ok(fn_ptr)
+        }
+
         #[allow(clippy::all)]
         #[allow(non_snake_case)]
+        #[cfg(not(test))]
         #[napi::bindgen_prelude::ctor]
         fn #module_register_name() {
-          unsafe fn cb(env: napi::bindgen_prelude::sys::napi_env) -> napi::bindgen_prelude::Result<napi::bindgen_prelude::sys::napi_value> {
-            let mut fn_ptr = std::ptr::null_mut();
-
-            napi::bindgen_prelude::check_status!(
-              napi::bindgen_prelude::sys::napi_create_function(
-                env,
-                #js_name.as_ptr() as *const _,
-                #name_len,
-                Some(#intermediate_ident),
-                std::ptr::null_mut(),
-                &mut fn_ptr,
-              ),
-              "Failed to register function `{}`",
-              #name_str,
-            )?;
-
-            Ok(fn_ptr)
-          }
-
-          napi::bindgen_prelude::register_module_export(#js_name, cb);
+          napi::bindgen_prelude::register_module_export(#js_mod_ident, #js_name, #cb_name);
         }
       }
     }
