@@ -4,7 +4,7 @@ use proc_macro2::{Ident, Literal, Span, TokenStream};
 use quote::ToTokens;
 
 use crate::{
-  codegen::{get_intermediate_ident, get_register_ident},
+  codegen::{get_intermediate_ident, get_register_ident, js_mod_to_token_stream},
   BindgenResult, FnKind, NapiImpl, NapiStruct, NapiStructKind, TryToTokens,
 };
 
@@ -239,17 +239,53 @@ impl NapiStruct {
     for field in self.fields.iter() {
       let field_js_name = &field.js_name;
       let ty = &field.ty;
-
+      let is_optional_field = if let syn::Type::Path(syn::TypePath {
+        path: syn::Path { segments, .. },
+        ..
+      }) = &ty
+      {
+        if let Some(last_path) = segments.last() {
+          last_path.ident == "Option"
+        } else {
+          false
+        }
+      } else {
+        false
+      };
       match &field.name {
         syn::Member::Named(ident) => {
           field_destructions.push(quote! { #ident });
-          obj_field_setters.push(quote! { obj.set(#field_js_name, #ident)?; });
-          obj_field_getters.push(quote! { let #ident: #ty = obj.get(#field_js_name)?.expect(&format!("Field {} should exist", #field_js_name)); });
+          if is_optional_field {
+            obj_field_setters.push(quote! {
+              if #ident.is_some() {
+                obj.set(#field_js_name, #ident)?;
+              }
+            });
+          } else {
+            obj_field_setters.push(quote! { obj.set(#field_js_name, #ident)?; });
+          }
+          if is_optional_field {
+            obj_field_getters.push(quote! { let #ident: #ty = obj.get(#field_js_name)?; });
+          } else {
+            obj_field_getters.push(quote! { let #ident: #ty = obj.get(#field_js_name)?.expect(&format!("Field {} should exist", #field_js_name)); });
+          }
         }
         syn::Member::Unnamed(i) => {
           field_destructions.push(quote! { arg#i });
-          obj_field_setters.push(quote! { obj.set(#field_js_name, arg#1)?; });
-          obj_field_getters.push(quote! { let arg#i: #ty = obj.get(#field_js_name)?.expect(&format!("Field {} should exist", #field_js_name)); });
+          if is_optional_field {
+            obj_field_setters.push(quote! {
+              if arg#1.is_some() {
+                obj.set(#field_js_name, arg#i)?;
+              }
+            });
+          } else {
+            obj_field_setters.push(quote! { obj.set(#field_js_name, arg#1)?; });
+          }
+          if is_optional_field {
+            obj_field_getters.push(quote! { let arg#i: #ty = obj.get(#field_js_name)?; });
+          } else {
+            obj_field_getters.push(quote! { let arg#i: #ty = obj.get(#field_js_name)?.expect(&format!("Field {} should exist", #field_js_name)); });
+          }
         }
       }
     }
@@ -378,7 +414,7 @@ impl NapiStruct {
   fn gen_register(&self) -> TokenStream {
     let name_str = self.name.to_string();
     let struct_register_name = get_register_ident(&format!("{}_struct", name_str));
-    let js_name = &self.js_name;
+    let js_name = format!("{}\0", self.js_name);
     let mut props = vec![];
 
     if self.kind == NapiStructKind::Constructor {
@@ -413,13 +449,14 @@ impl NapiStruct {
 
       props.push(prop);
     }
-
+    let js_mod_ident = js_mod_to_token_stream(self.js_mod.as_ref());
     quote! {
       #[allow(non_snake_case)]
       #[allow(clippy::all)]
+      #[cfg(not(test))]
       #[napi::bindgen_prelude::ctor]
       fn #struct_register_name() {
-        napi::bindgen_prelude::register_class(#name_str, #js_name, vec![#(#props),*]);
+        napi::bindgen_prelude::register_class(#name_str, #js_mod_ident, #js_name, vec![#(#props),*]);
       }
     }
   }
@@ -436,7 +473,7 @@ impl TryToTokens for NapiImpl {
 impl NapiImpl {
   fn gen_helper_mod(&self) -> BindgenResult<TokenStream> {
     let name_str = self.name.to_string();
-    let js_name = &self.js_name;
+    let js_name = format!("{}\0", self.js_name);
     let mod_name = Ident::new(
       &format!("__napi_impl_helper__{}", name_str),
       Span::call_site(),
@@ -478,7 +515,7 @@ impl NapiImpl {
     let mut props: Vec<_> = props.into_iter().collect();
     props.sort_by_key(|(_, prop)| prop.to_string());
     let props = props.into_iter().map(|(_, prop)| prop);
-
+    let js_mod_ident = js_mod_to_token_stream(self.js_mod.as_ref());
     Ok(quote! {
       #[allow(non_snake_case)]
       #[allow(clippy::all)]
@@ -486,9 +523,10 @@ impl NapiImpl {
         use super::*;
         #(#methods)*
 
+        #[cfg(not(test))]
         #[napi::bindgen_prelude::ctor]
         fn #register_name() {
-          napi::bindgen_prelude::register_class(#name_str, #js_name, vec![#(#props),*]);
+          napi::bindgen_prelude::register_class(#name_str, #js_mod_ident, #js_name, vec![#(#props),*]);
         }
       }
     })
